@@ -8,6 +8,9 @@
 #
 """ 对应WebQQ的数据结构抽象
 """
+import logging
+
+logger = logging.getLogger("twqq")
 
 
 class UniqueIds(object):
@@ -141,20 +144,40 @@ class Group(ObjectsBase):
 
         for item in data.get("cards", []):
             uin = item.get("muin")
-            self._uin_map[uin].card = item.get("card")
-            self._uin_name_map[item.get("card")] = uin
+            if uin in self._uin_map:
+                self._uin_map[uin].card = item.get("card")
+                self._uin_name_map[item.get("card")] = uin
+            else:
+                logger.warn(u"card info {0} not in map: {1!r}"
+                            .format(uin, item))
 
         for item in data.get("stats", []):
             uin = item.get("uin")
-            self._uin_map[uin].stat = item.get("stat")
-            self._uin_map[uin].client_type = item.get("client_type")
+            if uin in self._uin_map:
+                self._uin_map[uin].stat = item.get("stat")
+                self._uin_map[uin].client_type = item.get("client_type")
+            else:
+                logger.warn(u"stats info {0} not in map: {1!r}"
+                            .format(uin, item))
 
         for item in data.get("vipinfo", []):
             u = item.get("u")
-            self._uin_map[u].vipinfo = item
+            if u in self._uin_map:
+                self._uin_map[u].vipinfo = item
+            else:
+                logger.warn(u"vip info {0} not in map: {1!r}"
+                            .format(uin, item))
 
         self.set_detail_info(**data.get("ginfo", {}))
 
+    def __iter__(self):
+        """ 迭代返回成员的 uin
+        """
+        if hasattr(dict, "iterkeys"):
+            return self._uin_map.iterkeys()
+        else:
+            return self._uin_map.keys()
+
     def set_detail_info(self, face, memo, fingermemo, code, createtime,
                         flag, level, name, gid, owner, option, members, **kw):
         """ 组成员信息中对应 ginfo 中的元素
@@ -186,12 +209,17 @@ class Group(ObjectsBase):
 
         for item in members:
             uin = item.get("muin")
-            self._uin_map[uin].mflag = item.get("mflag")
+            tmp = self._uin_map.get(uin)
+            if tmp:
+                tmp.mflag = item.get("mflag")
 
     def __repr__(self):
         return u"<Group {0} have {1} members, Level {2}>"\
             .format(self.name, len(self._uin_map.keys()), self.level)
 
+    def __unicode__(self):
+        return self.__repr__()
+
     def get_nickname(self, uin):
         """ 获取群成员的昵称
         """
@@ -245,8 +273,16 @@ class GroupList(ObjectsBase):
     def __repr__(self):
         return str([x.name for x in self.gnamelist])
 
-    def __str__(self):
-        return self.__repr__()
+    def __unicode__(self):
+        return self.__repr__().decode("utf-8")
+
+    def __iter__(self):
+        """ 迭代返回群对象
+        """
+        if hasattr(dict, "itervalues"):
+            return self._gcode_map.itervalues()
+        else:
+            return self._gcode_map.values()
 
     @property
     def groups(self):
@@ -297,6 +333,9 @@ class GroupList(ObjectsBase):
         if item:
             return item.gid
 
+    def get_gcode(self, gid):
+        return self._gid_gcode_map.get(gid)
+
 
 class DiscuMemInfo(ObjectsBase):
 
@@ -341,8 +380,10 @@ class Discu(ObjectsBase):
             self._uin_map[item["uin"]] = DiscuMemInfo(**item)
 
         for item in mem_status:
-            self._uin_map[item["uin"]].status = item.get("status")
-            self._uin_map[item["uin"]].client_type = item.get("client_type")
+            tmp = self._uin_map.get(item["uin"])
+            if tmp:
+                tmp.status = item.get("status")
+                tmp.client_type = item.get("client_type")
 
     def get_mname(self, uin):
         item = self._uin_map.get(uin)
